@@ -40,6 +40,7 @@ from typing import (
     Iterable,
     List,
     Optional,
+    Set,
     Tuple,
     Type,
     Union,
@@ -105,6 +106,7 @@ class PdfReader(PdfDocCommon):
         password: Decrypt PDF file at initialization. If the
             password is None, the file will not be decrypted.
             Defaults to ``None``.
+
     """
 
     def __init__(
@@ -115,21 +117,24 @@ class PdfReader(PdfDocCommon):
     ) -> None:
         self.strict = strict
         self.flattened_pages: Optional[List[PageObject]] = None
+
         #: Storage of parsed PDF objects.
         self.resolved_objects: Dict[Tuple[Any, Any], Optional[PdfObject]] = {}
 
+        self._startxref: int = 0
         self.xref_index = 0
         self.xref: Dict[int, Dict[Any, Any]] = {}
         self.xref_free_entry: Dict[int, Dict[Any, Any]] = {}
         self.xref_objStm: Dict[int, Tuple[Any, Any]] = {}
         self.trailer = DictionaryObject()
 
-        # map page indirect_reference number to page number
+        # Map page indirect_reference number to page number
         self._page_id2num: Optional[Dict[Any, Any]] = None
 
         self._validated_root: Optional[DictionaryObject] = None
 
         self._initialize_stream(stream)
+        self._known_objects: Set[Tuple[int, int]] = set()
 
         self._override_encryption = False
         self._encryption: Optional[Encryption] = None
@@ -150,7 +155,6 @@ class PdfReader(PdfDocCommon):
             with open(stream, "rb") as fh:
                 stream = BytesIO(fh.read())
             self._stream_opened = True
-        self._startxref: int = 0
         self.read(stream)
         self.stream = stream
 
@@ -225,7 +229,16 @@ class PdfReader(PdfDocCommon):
                     self._validated_root = o
                     logger_warning(f"Root found at {o.indirect_reference!r}", __name__)
                     break
-            if self._validated_root is None:
+        if self._validated_root is None:
+            if not is_null_or_none(root) and "/Pages" in cast(DictionaryObject, cast(PdfObject, root).get_object()):
+                logger_warning(
+                    f"Possible root found at {cast(PdfObject, root).indirect_reference!r}, but missing /Catalog key",
+                    __name__
+                )
+                self._validated_root = cast(
+                    DictionaryObject, cast(PdfObject, root).get_object()
+                )
+            else:
                 raise PdfReadError("Cannot find Root object in pdf")
         return self._validated_root
 
@@ -236,17 +249,17 @@ class PdfReader(PdfDocCommon):
 
         Returns:
             /Info Dictionary; None if the entry does not exist
+
         """
         info = self.trailer.get(TK.INFO, None)
         if is_null_or_none(info):
             return None
-        else:
-            info = info.get_object()
-            if not isinstance(info, DictionaryObject):
-                raise PdfReadError(
-                    "Trailer not found or does not point to document information directory"
-                )
-            return info
+        info = info.get_object()
+        if not isinstance(info, DictionaryObject):
+            raise PdfReadError(
+                "Trailer not found or does not point to a document information dictionary"
+            )
+        return info
 
     @property
     def _ID(self) -> Optional[ArrayObject]:
@@ -255,39 +268,11 @@ class PdfReader(PdfDocCommon):
 
         Returns:
             /ID array; None if the entry does not exist
+
         """
         id = self.trailer.get(TK.ID, None)
         return None if is_null_or_none(id) else cast(ArrayObject, id.get_object())
 
-    def _repr_mimebundle_(
-        self,
-        include: Union[None, Iterable[str]] = None,
-        exclude: Union[None, Iterable[str]] = None,
-    ) -> Dict[str, Any]:
-        """
-        Integration into Jupyter Notebooks.
-
-        This method returns a dictionary that maps a mime-type to its
-        representation.
-
-        See https://ipython.readthedocs.io/en/stable/config/integrating.html
-        """
-        self.stream.seek(0)
-        pdf_data = self.stream.read()
-        data = {
-            "application/pdf": pdf_data,
-        }
-
-        if include is not None:
-            # Filter representations based on include list
-            data = {k: v for k, v in data.items() if k in include}
-
-        if exclude is not None:
-            # Remove representations based on exclude list
-            data = {k: v for k, v in data.items() if k not in exclude}
-
-        return data
-
     @property
     def pdf_header(self) -> str:
         """
@@ -324,6 +309,7 @@ class PdfReader(PdfDocCommon):
 
         Returns:
             Page number or None.
+
         """
         if self._page_id2num is None:
             self._page_id2num = {
@@ -338,8 +324,7 @@ class PdfReader(PdfDocCommon):
         else:
             idnum = indirect_reference.idnum
         assert self._page_id2num is not None, "hint for mypy"
-        ret = self._page_id2num.get(idnum, None)
-        return ret
+        return self._page_id2num.get(idnum, None)
 
     def _get_object_from_stream(
         self, indirect_reference: IndirectObject
@@ -367,7 +352,7 @@ class PdfReader(PdfDocCommon):
                 raise PdfReadError("Object is in wrong index.")
             stream_data.seek(int(obj_stm["/First"] + offset), 0)  # type: ignore
 
-            # to cope with some case where the 'pointer' is on a white space
+            # To cope with case where the 'pointer' is on a white space
             read_non_whitespace(stream_data)
             stream_data.seek(-1, 1)
 
@@ -428,7 +413,7 @@ class PdfReader(PdfDocCommon):
                     idnum != indirect_reference.idnum
                     or generation != indirect_reference.generation
                 ):
-                    raise PdfReadError("not matching, we parse the file for it")
+                    raise PdfReadError("Not matching, we parse the file for it")
             except Exception:
                 if hasattr(self.stream, "getbuffer"):
                     buf = bytes(self.stream.getbuffer())
@@ -455,7 +440,7 @@ class PdfReader(PdfDocCommon):
                     idnum = -1
                     generation = -1  # exception will be raised below
             if idnum != indirect_reference.idnum and self.xref_index:
-                # Xref table probably had bad indexes due to not being zero-indexed
+                # xref table probably had bad indexes due to not being zero-indexed
                 if self.strict:
                     raise PdfReadError(
                         f"Expected object ID ({indirect_reference.idnum} {indirect_reference.generation}) "
@@ -466,13 +451,18 @@ class PdfReader(PdfDocCommon):
             elif idnum != indirect_reference.idnum and self.strict:
                 # some other problem
                 raise PdfReadError(
-                    f"Expected object ID ({indirect_reference.idnum} "
-                    f"{indirect_reference.generation}) does not match actual "
-                    f"({idnum} {generation})."
+                    f"Expected object ID ({indirect_reference.idnum} {indirect_reference.generation}) "
+                    f"does not match actual ({idnum} {generation})."
                 )
             if self.strict:
                 assert generation == indirect_reference.generation
+
+            current_object = (indirect_reference.idnum, indirect_reference.generation)
+            if current_object in self._known_objects:
+                raise PdfReadError(f"Detected loop with self reference for {indirect_reference!r}.")
+            self._known_objects.add(current_object)
             retval = read_object(self.stream, self)  # type: ignore
+            self._known_objects.remove(current_object)
 
             # override encryption is used for the /Encrypt dictionary
             if not self._override_encryption and self._encryption is not None:
@@ -536,11 +526,10 @@ class PdfReader(PdfDocCommon):
     def read_object_header(self, stream: StreamType) -> Tuple[int, int]:
         # Should never be necessary to read out whitespace, since the
         # cross-reference table should put us in the right spot to read the
-        # object header. In reality some files have stupid cross reference
+        # object header. In reality some files have stupid cross-reference
         # tables that are off by whitespace bytes.
-        extra = False
         skip_over_comment(stream)
-        extra |= skip_over_whitespace(stream)
+        extra = skip_over_whitespace(stream)
         stream.seek(-1, 1)
         idnum = read_until_whitespace(stream)
         extra |= skip_over_whitespace(stream)
@@ -583,7 +572,7 @@ class PdfReader(PdfDocCommon):
         return obj
 
     def _replace_object(self, indirect: IndirectObject, obj: PdfObject) -> PdfObject:
-        # function reserved for future dev
+        # function reserved for future development
         if indirect.pdf != self:
             raise ValueError("Cannot update PdfReader with external object")
         if (indirect.generation, indirect.idnum) not in self.resolved_objects:
@@ -598,20 +587,21 @@ class PdfReader(PdfDocCommon):
 
         Args:
             stream: The PDF file stream.
+
         """
         self._basic_validation(stream)
         self._find_eof_marker(stream)
         startxref = self._find_startxref_pos(stream)
         self._startxref = startxref
 
-        # check and eventually correct the startxref only in not strict
+        # check and eventually correct the startxref only if not strict
         xref_issue_nr = self._get_xref_issues(stream, startxref)
         if xref_issue_nr != 0:
             if self.strict and xref_issue_nr:
                 raise PdfReadError("Broken xref table")
             logger_warning(f"incorrect startxref pointer({xref_issue_nr})", __name__)
 
-        # read all cross reference tables and their trailers
+        # read all cross-reference tables and their trailers
         self._read_xref_tables_and_trailers(stream, startxref, xref_issue_nr)
 
         # if not zero-indexed, verify that the table is correct; change it if necessary
@@ -622,7 +612,7 @@ class PdfReader(PdfDocCommon):
                     continue
                 xref_k = sorted(
                     xref_entry.keys()
-                )  # must ensure ascendant to prevent damage
+                )  # ensure ascending to prevent damage
                 for id in xref_k:
                     stream.seek(xref_entry[id], 0)
                     try:
@@ -666,14 +656,13 @@ class PdfReader(PdfDocCommon):
             raise UnsupportedOperation("cannot read header")
         if header_byte == b"":
             raise EmptyFileError("Cannot read an empty file")
-        elif header_byte != b"%PDF-":
+        if header_byte != b"%PDF-":
             if self.strict:
                 raise PdfReadError(
                     f"PDF starts with '{header_byte.decode('utf8')}', "
                     "but '%PDF-' expected"
                 )
-            else:
-                logger_warning(f"invalid pdf header: {header_byte}", __name__)
+            logger_warning(f"invalid pdf header: {header_byte}", __name__)
         stream.seek(0, os.SEEK_END)
 
     def _find_eof_marker(self, stream: StreamType) -> None:
@@ -687,7 +676,7 @@ class PdfReader(PdfDocCommon):
         HEADER_SIZE = 8  # to parse whole file, Header is e.g. '%PDF-1.6'
         line = b""
         first = True
-        while line[:5] != b"%%EOF":
+        while not line.startswith(b"%%EOF"):
             if line != b"" and first:
                 if any(
                     line.strip().endswith(tr) for tr in (b"%%EO", b"%%E", b"%%", b"%")
@@ -706,8 +695,7 @@ class PdfReader(PdfDocCommon):
             if stream.tell() < HEADER_SIZE:
                 if self.strict:
                     raise PdfReadError("EOF marker not found")
-                else:
-                    logger_warning("EOF marker not found", __name__)
+                logger_warning("EOF marker not found", __name__)
             line = read_previous_line(stream)
 
     def _find_startxref_pos(self, stream: StreamType) -> int:
@@ -719,6 +707,7 @@ class PdfReader(PdfDocCommon):
 
         Returns:
             The bytes offset
+
         """
         line = read_previous_line(stream)
         try:
@@ -731,7 +720,7 @@ class PdfReader(PdfDocCommon):
             logger_warning("startxref on same line as offset", __name__)
         else:
             line = read_previous_line(stream)
-            if line[:9] != b"startxref":
+            if not line.startswith(b"startxref"):
                 raise PdfReadError("startxref not found")
         return startxref
 
@@ -771,6 +760,8 @@ class PdfReader(PdfDocCommon):
             cnt = 0
             while cnt < size:
                 line = stream.read(20)
+                if not line:
+                    raise PdfReadError("Unexpected empty line in Xref table.")
 
                 # It's very clear in section 3.4.3 of the PDF spec
                 # that all cross-reference table lines are a fixed
@@ -797,7 +788,6 @@ class PdfReader(PdfDocCommon):
 
                     offset, generation = int(offset_b), int(generation_b)
                 except Exception:
-                    # if something wrong occurred
                     if hasattr(stream, "getbuffer"):
                         buf = bytes(stream.getbuffer())
                     else:
@@ -806,7 +796,7 @@ class PdfReader(PdfDocCommon):
                         buf = stream.read(-1)
                         stream.seek(p)
 
-                    f = re.search(f"{num}\\s+(\\d+)\\s+obj".encode(), buf)
+                    f = re.search(rf"{num}\s+(\d+)\s+obj".encode(), buf)
                     if f is None:
                         logger_warning(
                             f"entry {num} in Xref table invalid; object not found",
@@ -884,8 +874,7 @@ class PdfReader(PdfDocCommon):
                             f"Previous trailer cannot be read: {e.args}", __name__
                         )
                         break
-                    else:
-                        raise PdfReadError(f"Trailer cannot be read: {e.args}")
+                    raise PdfReadError(f"Trailer cannot be read: {e!s}")
                 self._process_xref_stream(xrefstream)
                 if "/Prev" in xrefstream:
                     startxref = cast(int, xrefstream["/Prev"])
@@ -929,10 +918,8 @@ class PdfReader(PdfDocCommon):
                 )
             stream.seek(p, 0)
         if "/Prev" in new_trailer:
-            startxref = new_trailer["/Prev"]
-            return startxref
-        else:
-            return None
+            return new_trailer["/Prev"]
+        return None
 
     def _read_xref_other_error(
         self, stream: StreamType, startxref: int
@@ -972,7 +959,7 @@ class PdfReader(PdfDocCommon):
                 self._rebuild_xref_table(stream)
                 return None
             except Exception:
-                raise PdfReadError("can not rebuild xref")
+                raise PdfReadError("Cannot rebuild xref")
         raise PdfReadError("Could not find xref table at specified location")
 
     def _read_pdf15_xref_stream(
@@ -982,11 +969,12 @@ class PdfReader(PdfDocCommon):
         stream.seek(-1, 1)
         idnum, generation = self.read_object_header(stream)
         xrefstream = cast(ContentStream, read_object(stream, self))
-        assert cast(str, xrefstream["/Type"]) == "/XRef"
+        if cast(str, xrefstream["/Type"]) != "/XRef":
+            raise PdfReadError(f"Unexpected type {xrefstream['/Type']!r}")
         self.cache_indirect_object(generation, idnum, xrefstream)
         stream_data = BytesIO(xrefstream.get_data())
-        # Index pairs specify the subsections in the dictionary. If
-        # none create one subsection that spans everything.
+        # Index pairs specify the subsections in the dictionary.
+        # If none, create one subsection that spans everything.
         idx_pairs = xrefstream.get("/Index", [0, xrefstream.get("/Size")])
         entry_sizes = cast(Dict[Any, Any], xrefstream.get("/W"))
         assert len(entry_sizes) >= 3
@@ -1004,8 +992,7 @@ class PdfReader(PdfDocCommon):
             # W array indicates...the default value shall be used
             if i == 0:
                 return 1  # First value defaults to 1
-            else:
-                return 0
+            return 0
 
         def used_before(num: int, generation: Union[int, Tuple[int, ...]]) -> bool:
             # We move backwards through the xrefs, don't replace any.
@@ -1026,7 +1013,11 @@ class PdfReader(PdfDocCommon):
 
         Returns:
             0 means no issue, other values represent specific issues.
+
         """
+        if startxref == 0:
+            return 4
+
         stream.seek(startxref - 1, 0)  # -1 to check character before
         line = stream.read(1)
         if line == b"j":
@@ -1035,7 +1026,7 @@ class PdfReader(PdfDocCommon):
             return 1
         line = stream.read(4)
         if line != b"xref":
-            # not an xref so check if it is an XREF object
+            # not a xref so check if it is an XREF object
             line = b""
             while line in b"0123456789 \t":
                 line = stream.read(1)
@@ -1089,7 +1080,7 @@ class PdfReader(PdfDocCommon):
                             f" whereas {o.get('/N')} expected",
                             __name__,
                         )
-                except Exception:  # could be of many cause
+                except Exception:  # could be multiple causes
                     pass
 
         stream.seek(0, 0)
@@ -1160,6 +1151,7 @@ class PdfReader(PdfDocCommon):
         Returns:
             An indicator if the document was decrypted and whether it was the
             owner password or the user password.
+
         """
         if not self._encryption:
             raise PdfReadError("Not encrypted file")
@@ -1185,6 +1177,7 @@ class PdfReader(PdfDocCommon):
 
         Returns:
             The created object. ``None`` means no object was created.
+
         """
         catalog = self.root_object
 
@@ -1194,7 +1187,7 @@ class PdfReader(PdfDocCommon):
             return None
         acroform = cast(DictionaryObject, catalog[NameObject("/AcroForm")])
         if "/Fields" not in acroform:
-            # TODO: :No error returns but may be extended for XFA Forms
+            # TODO: No error but this may be extended for XFA Forms
             return None
 
         interim = DictionaryObject()
@@ -1202,7 +1195,7 @@ class PdfReader(PdfDocCommon):
         interim[NameObject("/Kids")] = acroform[NameObject("/Fields")]
         self.cache_indirect_object(
             0,
-            max([i for (g, i) in self.resolved_objects if g == 0]) + 1,
+            max(i for (g, i) in self.resolved_objects if g == 0) + 1,
             interim,
         )
         arr = ArrayObject()
@@ -1227,6 +1220,7 @@ class PdfReader(PdfDocCommon):
 
         Returns:
             The modified object. ``None`` means no object was modified.
+
         """
         catalog = self.root_object
 
@@ -1244,3 +1238,34 @@ class PdfReader(PdfDocCommon):
         )
         interim[NameObject("/T")] = TextStringObject(name)
         return interim
+
+    def _repr_mimebundle_(
+        self,
+        include: Union[None, Iterable[str]] = None,
+        exclude: Union[None, Iterable[str]] = None,
+    ) -> Dict[str, Any]:
+        """
+        Integration into Jupyter Notebooks.
+
+        This method returns a dictionary that maps a mime-type to its
+        representation.
+
+        .. seealso::
+
+            https://ipython.readthedocs.io/en/stable/config/integrating.html
+        """
+        self.stream.seek(0)
+        pdf_data = self.stream.read()
+        data = {
+            "application/pdf": pdf_data,
+        }
+
+        if include is not None:
+            # Filter representations based on include list
+            data = {k: v for k, v in data.items() if k in include}
+
+        if exclude is not None:
+            # Remove representations based on exclude list
+            data = {k: v for k, v in data.items() if k not in exclude}
+
+        return data
