@@ -8,7 +8,7 @@ from itertools import product as cartesian_product
 from pathlib import Path
 
 import pytest
-from PIL import Image
+from PIL import Image, ImageOps
 
 from pypdf import PdfReader
 from pypdf.errors import DeprecationError, PdfReadError
@@ -27,7 +27,6 @@ from .test_encryption import HAS_AES
 from .test_images import image_similarity
 
 filter_inputs = (
-    # "", '', """""",
     string.ascii_lowercase,
     string.ascii_uppercase,
     string.ascii_letters,
@@ -76,8 +75,7 @@ def test_flatedecode_unsupported_predictor():
 def test_flate_decode_decompress_with_array_params(params):
     """FlateDecode decode() method works correctly with array parameters."""
     codec = FlateDecode()
-    s = ""
-    s = s.encode()
+    s = b""
     encoded = codec.encode(s)
     with pytest.raises(DeprecationError):
         assert codec.decode(encoded, params) == s
@@ -106,6 +104,8 @@ def test_flate_decode_decompress_with_array_params(params):
         ),  # Same as previous, but whitespaced
         ("30313233343536373839616263646566414243444546>", string.hexdigits.encode()),
         ("20090a0d0b0c>", string.whitespace.encode()),
+        # Odd number of hexadecimal digits behaves as if a 0 (zero) followed the last digit
+        ("3938373635343332313>", string.digits[::-1].encode()),
     ],
     ids=[
         "empty",
@@ -116,16 +116,13 @@ def test_flate_decode_decompress_with_array_params(params):
         "digits_whitespace",
         "hexdigits",
         "whitespace",
+        "odd_number",
     ],
 )
 def test_ascii_hex_decode_method(data, expected):
     """
     Feeds a bunch of values to ASCIIHexDecode.decode() and ensures the
     correct output is returned.
-
-    TODO What is decode() supposed to do for such inputs as ">>", ">>>" or
-    any other not terminated by ">"? (For the latter case, an exception
-    is currently raised.)
     """
     assert ASCIIHexDecode.decode(data) == expected
 
@@ -169,7 +166,7 @@ def test_ascii85decode_five_zero_bytes():
     """
     ASCII85Decode handles the special case of five zero bytes correctly.
 
-    From ISO 32000 (2008) §7.4.3:
+    ISO 32000-1:2008 §7.4.3:
 
     «As a special case, if all five bytes are 0, they shall be represented by
     the character with code 122 (z) instead of by five exclamation points
@@ -189,7 +186,7 @@ def test_ascii85decode_five_zero_bytes():
 
 
 def test_ccitparameters():
-    with pytest.raises(
+    with pytest.warns(
         DeprecationWarning,
         match="CCITParameters is deprecated and will be removed in pypdf 6.0.0. Use CCITTParameters instead",
     ):
@@ -355,13 +352,13 @@ def test_png_transparency_reverse():
     """Cf issue #1599"""
     pdf_path = RESOURCE_ROOT / "labeled-edges-center-image.pdf"
     reader = PdfReader(pdf_path)
-    _refimg = Image.open(
+    refimg = Image.open(
         BytesIO(get_data_from_url(name="labeled-edges-center-image.png"))
     )
     data = reader.pages[0].images[0]
-    _img = Image.open(BytesIO(data.data))
+    img = Image.open(BytesIO(data.data))
     assert ".jp2" in data.name
-    # assert list(img.getdata()) == list(refimg.getdata())
+    assert list(img.getdata()) == list(refimg.getdata())
 
 
 @pytest.mark.enable_socket
@@ -376,9 +373,8 @@ def test_iss1787():
     obj = data.indirect_reference.get_object()
     obj["/DecodeParms"][NameObject("/Columns")] = NumberObject(1000)
     obj.decoded_self = None
-    with pytest.raises(PdfReadError) as exc:
-        reader.pages[0].images[0]
-    assert exc.value.args[0] == "Image data is not rectangular"
+    with pytest.raises(expected_exception=PdfReadError, match="^Unsupported PNG filter 244$"):
+        _ = reader.pages[0].images[0]
 
 
 @pytest.mark.enable_socket
@@ -394,7 +390,7 @@ def test_tiff_predictor():
 
 @pytest.mark.enable_socket
 def test_rgba():
-    """Decode rgb with transparency"""
+    """Decode RGB with transparency"""
     with PILContext():
         reader = PdfReader(BytesIO(get_data_from_url(name="tika-972174.pdf")))
         data = reader.pages[0].images[0]
@@ -407,7 +403,7 @@ def test_rgba():
 
 @pytest.mark.enable_socket
 def test_cmyk():
-    """Decode cmyk"""
+    """Decode CMYK"""
     # JPEG compression
     try:
         from Crypto.Cipher import AES  # noqa: F401
@@ -475,7 +471,7 @@ def test_index_lookup():
     assert data.image.mode == "RGB"
     assert image_similarity(data.image, refimg) > 0.999
     # indexed CMYK images
-    # currently with a  TODO as we convert to RBG the palette
+    # currently with a TODO as we convert the palette to RGB
     reader = PdfReader(BytesIO(get_data_from_url(name="tika-972174.pdf")))
     refimg = Image.open(BytesIO(get_data_from_url(name="usa.png")))
     data = reader.pages[0].images["/Im3"]
@@ -644,3 +640,52 @@ def test_ascii85decode__non_recoverable(caplog):
     with pytest.raises(ValueError, match="Non-Ascii85 digit found: Ã"):
         ASCII85Decode.decode(data)
     assert caplog.text == ""
+
+
+@pytest.mark.enable_socket
+def test_ccitt_fax_decode__black_is_1():
+    url = "https://github.com/user-attachments/files/19288881/imagemagick-CCITTFaxDecode_BlackIs1-true.pdf"
+    name = "issue3193.pdf"
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
+    other_reader = PdfReader(RESOURCE_ROOT / "imagemagick-CCITTFaxDecode.pdf")
+
+    actual_image = reader.pages[0].images[0].image
+    expected_image_inverted = other_reader.pages[0].images[0].image
+    expected_pixels = list(ImageOps.invert(expected_image_inverted).getdata())
+    actual_pixels = list(actual_image.getdata())
+    assert expected_pixels == actual_pixels
+
+
+@pytest.mark.enable_socket
+def test_flate_decode__image_is_none_due_to_size_limit(caplog):
+    url = "https://github.com/user-attachments/files/19464256/file.pdf"
+    name = "issue3220.pdf"
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
+    images = reader.pages[0].images
+    assert len(images) == 1
+    image = images[0]
+    assert image.name == "Im0.png"
+    assert image.image is None
+    assert (
+        "Failed loading image: Image size (180000000 pixels) exceeds limit of "
+        "178956970 pixels, could be decompression bomb DOS attack."
+    ) in caplog.messages
+
+
+@pytest.mark.enable_socket
+def test_flate_decode__not_rectangular(caplog):
+    url = "https://github.com/user-attachments/files/19663603/issue3241_compressed.txt"
+    name = "issue3241.txt"
+    data = get_data_from_url(url, name=name)
+    decode_parms = DictionaryObject()
+    decode_parms[NameObject("/Predictor")] = NumberObject(15)
+    decode_parms[NameObject("/Columns")] = NumberObject(4881)
+    actual = FlateDecode.decode(data=data, decode_parms=decode_parms)
+    actual_image = BytesIO()
+    Image.frombytes(mode="1", size=(4881, 81), data=actual).save(actual_image, format="png")
+
+    url = "https://github.com/user-attachments/assets/c5695850-c076-4255-ab72-7c86851a4a04"
+    name = "issue3241.png"
+    expected = get_data_from_url(url, name=name)
+    assert actual_image.getvalue() == expected
+    assert caplog.messages == ["Image data is not rectangular. Adding padding."]
