@@ -14,7 +14,7 @@ import pytest
 from PIL import Image, ImageChops
 from PIL import __version__ as pil_version
 
-from pypdf import PdfReader, PdfWriter
+from pypdf import PdfReader, PdfWriter, Transformation
 from pypdf.constants import PageAttributes as PG
 from pypdf.errors import PdfReadError, PdfReadWarning
 from pypdf.generic import (
@@ -112,10 +112,7 @@ def test_pdfreader_file_load():
             assert expected_line == actual_line
 
         pdftext = pdftext.replace(b"\r\n", b"\n")  # fix for windows
-        assert text == pdftext, (
-            "PDF extracted text differs from expected value.\n\n"
-            "Expected:\n\n%r\n\nExtracted:\n\n%r\n\n" % (pdftext, text)
-        )
+        assert text == pdftext
 
 
 def test_pdfreader_jpeg_image():
@@ -138,11 +135,7 @@ def test_pdfreader_jpeg_image():
         data = x_object["/Im4"].get_data()
 
         # Compare the text of the PDF to a known source
-        assert binascii.hexlify(data).decode() == imagetext, (
-            "PDF extracted image differs from expected value.\n\n"
-            "Expected:\n\n%r\n\nExtracted:\n\n%r\n\n"
-            % (imagetext, binascii.hexlify(data).decode())
-        )
+        assert binascii.hexlify(data).decode() == imagetext
 
 
 def test_decrypt():
@@ -272,6 +265,24 @@ def test_extract_textbench(enable, url, pages, print_result=False):
         pass
 
 
+def test_transform_compress_identical_objects():
+    reader = PdfReader(RESOURCE_ROOT / "two-different-pages.pdf")
+    writer = PdfWriter()
+
+    for page in reader.pages:
+        op = Transformation().scale(sx=0.8, sy=0.8)
+        page.add_transformation(op)
+        writer.add_page(page)
+    writer.compress_identical_objects()
+    bytes_out = BytesIO()
+    writer.write(bytes_out)
+    result_reader = PdfReader(bytes_out)
+    pg1_text = result_reader.pages[0].extract_text()
+    pg2_text = result_reader.pages[1].extract_text()
+    assert pg1_text.strip() == "1"
+    assert pg2_text.strip() == "2"
+
+
 @pytest.mark.slow
 def test_orientations():
     p = PdfReader(RESOURCE_ROOT / "test Orient.pdf").pages[0]
@@ -1035,7 +1046,7 @@ def test_inline_images():
 
 
 @pytest.mark.enable_socket
-def test_iss():
+def test_issue1899():
     url = "https://github.com/py-pdf/pypdf/files/11801077/lv2018tconv.pdf"
     name = "lv2018tconv.pdf"
     reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
